import 'package:flutter/material.dart';
// import 'package:http/http.dart' as http;  ← 사용안함. Api.serverRequest() 사용
import 'dart:convert' show utf8, jsonEncode;
import 'package:crypto/crypto.dart';
import 'package:shared_preferences/shared_preferences.dart';

// 우리의 Api 모듈
import '../../plugins/api.dart';

// 안내 모달창
import '../../dialogs/response_dialog.dart';

// (기존) ID/PW 찾기, 회원가입 페이지
import 'id_finding_page.dart';
import 'pw_finding_page.dart';
import 'signup_page.dart';

// 구글 로그인
import 'package:google_sign_in/google_sign_in.dart';
import 'package:firebase_auth/firebase_auth.dart';

// 광고
import 'package:google_mobile_ads/google_mobile_ads.dart';

// 메인 페이지
import '../room/main_page.dart';

// 설정
import '../../config/config.dart';

// 뒤로가기
import 'package:fluttertoast/fluttertoast.dart'; // 뒤로가기 안내 문구에 Toast 등 사용

class LoginPage extends StatefulWidget {
  const LoginPage({Key? key}) : super(key: key);

  @override
  State<LoginPage> createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  // ─────────────────────────────────────────
  // (A) ID/PW 관련
  // ─────────────────────────────────────────
  final TextEditingController idController = TextEditingController();
  final TextEditingController passwordController = TextEditingController();
  bool autoLogin = false;
  String loginErrorMessage = ''; // 로그인 실패 시 안내

  // ─────────────────────────────────────────
  // (B) 구글 로그인 객체
  // ─────────────────────────────────────────
  final GoogleSignIn _googleSignIn = GoogleSignIn(
    scopes: <String>['email'],
  );

  // ─────────────────────────────────────────
  // (C) 광고 배너
  // ─────────────────────────────────────────
  BannerAd? _bannerAd;
  bool _isBannerReady = false;
  String adUnitId = Config.testAdUnitId;

  // 뒤로가기 처리
  DateTime? _lastPressedTime;

  // 로딩 중
  bool _isLoading = false;

  // 예: 2초 이내로 뒤로가기를 한 번 더 누르면 종료
  static const _exitDuration = Duration(seconds: 2);

  Future<bool> _onWillPop() async {
    final now = DateTime.now();
    if (_lastPressedTime == null ||
        now.difference(_lastPressedTime!) > _exitDuration) {
      // 첫 번째 뒤로가기 누름 or 이전 누름이 오래 전
      _lastPressedTime = now;

      // 안내 문구 띄우기 (Toast 예시)
      Fluttertoast.showToast(
        msg: '한 번 더 누르면 앱이 종료됩니다.',
        toastLength: Toast.LENGTH_SHORT,
        gravity: ToastGravity.BOTTOM,
      );
      return false; // 페이지 pop 하지 않음
    }
    // 2초 이내에 뒤로가기 두 번째 누름 → 앱 종료 허용
    return true; // pop 허용 (Scaffold 밖으로 벗어남, 결과적으로 앱 종료)
  }

  @override
  void initState() {
    super.initState();
    _initBannerAd();
  }

  void _initBannerAd() {
    _bannerAd = BannerAd(
      // 실제/테스트 배너 광고 단위 ID
      adUnitId: adUnitId,
      size: AdSize.banner,
      request: const AdRequest(),
      listener: BannerAdListener(
        onAdLoaded: (ad) {
          setState(() => _isBannerReady = true);
        },
        onAdFailedToLoad: (ad, error) {
          ad.dispose();
        },
      ),
    );
    _bannerAd?.load();
  }

  @override
  void dispose() {
    _bannerAd?.dispose();
    super.dispose();
  }

  // ─────────────────────────────────────────
  // (D1) ID/PW 로그인
  // ─────────────────────────────────────────
  Future<void> _loginWithIdPw() async {
    setState(() => _isLoading = true);
    final id = idController.text.trim();
    final pw = passwordController.text.trim();

    // PW SHA-256 해싱
    final bytes = utf8.encode(pw);
    final digest = sha256.convert(bytes);
    final hashedPw = digest.toString();

    final requestBody = {
      "user_id": id,
      "user_pw": hashedPw,
    };

    try {
      // (1) /user/login 서버 요청
      final response = await Api.serverRequest(uri: '/user/login', body: requestBody);

      if (response['result'] == 'OK') {
        // 내부 응답
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          // 로그인 성공
          print('ID/PW 로그인 성공: $resp');

          // (a) google_user_yn = N
          final prefs = await SharedPreferences.getInstance();
          await prefs.setString('oauth_type', 'idpw');
          await prefs.setBool('auto_login', true);
          await prefs.setString('jwt_token', resp['auth']['token'].toString());
          await prefs.setInt('my_user_seq', resp['auth']['user_seq']);

          // 메인 페이지 이동
          if (!mounted) return;
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (_) => const MainPage()),
          );
        } else {
          showResponseDialog(context, resp['response_info']['msg_title'], resp['response_info']['msg_content']);
        }
      } else {
        // 서버 통신 자체가 FAIL
        showResponseDialog(context, '오류', '로그인 요청 실패');
      }
    } catch (e) {
      showResponseDialog(context, '오류', '로그인 요청 중 예외 발생.\n$e');
    } finally {
      setState(() => _isLoading = false);
    }
  }

  // ─────────────────────────────────────────
  // (D2) 구글 로그인
  // ─────────────────────────────────────────
  Future<void> _googleLogin() async {
    setState(() => _isLoading = true);
    try {
      // 1) 구글 계정 선택
      final GoogleSignInAccount? googleUser = await _googleSignIn.signIn();
      if (googleUser == null) {
        // 사용자가 로그인 창에서 취소 누름
        return;
      }

      // 2) 구글 인증 정보 가져오기
      final GoogleSignInAuthentication googleAuth = await googleUser.authentication;

      // 3) FirebaseAuth Credential 생성
      final AuthCredential credential = GoogleAuthProvider.credential(
        accessToken: googleAuth.accessToken,
        idToken: googleAuth.idToken,
      );

      // 4) FirebaseAuth로 로그인
      final UserCredential userCredential =
          await FirebaseAuth.instance.signInWithCredential(credential);

      final User? user = userCredential.user;
      if (user == null) {
        print('구글 로그인 실패: Firebase User가 null');
        _showAlert('로그인 오류', 'Firebase User가 null입니다.');
        return;
      }

      final idToken = await user.getIdToken();

      // 서버에 구글 로그인 정보 전송
      final requestBody = {
        'id_token': idToken,
      };

      final response = await Api.serverRequest(uri: '/user/google/login', body: requestBody);
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          final prefs = await SharedPreferences.getInstance();
          await prefs.setString('oauth_type', 'google');
          await prefs.setBool('auto_login', true);
          await prefs.setString('jwt_token', resp['auth']['token'].toString());
          await prefs.setInt('my_user_seq', resp['auth']['user_seq']);
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (_) => const MainPage()),
          );
        } else {
          showResponseDialog(context, resp['response_info']['msg_title'], resp['response_info']['msg_content']);
        }
      } else {
        showResponseDialog(context, '오류', '구글 로그인 요청 실패');
      }

      // (선택) SharedPreferences에 google_user_yn = 'Y' 저장 등
      final prefs = await SharedPreferences.getInstance();
      await prefs.setString('google_user_yn', 'Y');
    } catch (e) {
      _showAlert('오류', '구글 로그인 중 오류가 발생했습니다.\n$e');
    } finally {
      setState(() => _isLoading = false);
    }
  }
  
  // ─────────────────────────────────────────
  // (D3) 구글 회원가입
  // ─────────────────────────────────────────
  Future<void> _googleSignUp() async {
    final agreed = await _showTermsModal();
    if (agreed != true) {
      return;
    }

    try {
    // (2) 구글 계정 선택
    final GoogleSignInAccount? googleUser = await _googleSignIn.signIn();
    if (googleUser == null) {
      // 사용자가 회원가입 창에서 취소 누름
      return;
    }

    // (3) 구글 인증 정보 가져오기
    final GoogleSignInAuthentication googleAuth = await googleUser.authentication;

    // (4) FirebaseAuth Credential 생성
    final AuthCredential credential = GoogleAuthProvider.credential(
      accessToken: googleAuth.accessToken,
      idToken: googleAuth.idToken,
    );

    // (5) FirebaseAuth로 로그인 (회원가입 시도)
    // 사실 "회원가입"이라는 명칭을 썼지만, Firebase 쪽에선 같은 signInWithCredential()를 사용
    final UserCredential userCredential =
        await FirebaseAuth.instance.signInWithCredential(credential);

    final User? user = userCredential.user;
    if (user == null) {
      showResponseDialog(context, '오류', '구글계정 인증에 실패했습니다.');
      return;
    }

    // (6) idToken 추출 후, 서버에 회원가입 요청
    final idToken = await user.getIdToken();
    final requestBody = {
      'id_token': idToken,
    };

    // 서버 측 '/user/google/signup' API 호출 (예시)
    final response = await Api.serverRequest(uri: '/user/google/signup', body: requestBody);
    if (response['result'] == 'OK') {
      final resp = response['response'] ?? {};
      if (resp['result'] == 'OK') {
        // 회원가입 성공 안내
        showResponseDialog(context, '회원가입 완료', '구글 회원가입이 완료되었습니다.');
      } else {
        // 서버 응답 자체가 OK가 아니면, 어떤 에러 메시지를 띄울지 처리
        final msgTitle = resp['response_info']?['msg_title'] ?? '오류';
        final msgContent = resp['response_info']?['msg_content'] ?? '회원가입에 실패했습니다.';
        showResponseDialog(context, msgTitle, msgContent);
      }
    } else {
      // 서버 통신이 FAIL (응답 result != OK)
      showResponseDialog(context, '오류', '구글 회원가입 요청 실패');
    }

    } catch (e) {
      showResponseDialog(context, '오류', '구글 회원가입 중 오류가 발생했습니다.\n$e');
    }
  }

  // ─────────────────────────────────────────
  // (E) 약관 모달
  // ─────────────────────────────────────────
  Future<bool?> _showTermsModal() async {
    return showDialog<bool>(
      context: context,
      barrierDismissible: false,
      builder: (ctx) {
        return AlertDialog(
          backgroundColor: Colors.white,
          title: const Text(
            '개인정보 수집 및 이용 동의서',
            style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
          ),
          content: SingleChildScrollView(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: const [
                Text(
                  '''올스코어(이하 "회사"라 합니다)는 이용자의 개인정보를 중요시하며, 「개인정보 보호법」 등 관련 법령을 준수하고 있습니다. 회사는 개인정보 수집 및 이용에 관한 사항을 아래와 같이 안내드리오니, 내용을 충분히 숙지하신 후 동의하여 주시기 바랍니다.

1. 수집하는 개인정보 항목
필수 항목: 아이디(ID), 비밀번호(PW), 닉네임(실명 아님), 이메일 주소
선택 항목: 소속, 자기소개

2. 개인정보의 수집 및 이용 목적
회원 관리
회원 식별 및 인증
부정 이용 방지 및 비인가 사용 방지
서비스 이용에 따른 문의 사항 처리
서비스 제공
게임 방 생성 및 참여 등 기본 서비스 제공
통계 및 순위 제공 등 부가 서비스 제공
고객 지원 및 공지사항 전달
서비스 관련 중요한 공지사항 전달
이용자 문의 및 불만 처리

3. 개인정보의 보유 및 이용 기간
회원 탈퇴 시: 수집된 모든 개인정보는 회원 탈퇴 즉시 파기합니다.
관련 법령에 따른 보관: 전자상거래 등에서의 소비자 보호에 관한 법률 등 관계 법령의 규정에 따라 일정 기간 보관이 필요한 경우 해당 기간 동안 보관합니다.
계약 또는 청약 철회 등에 관한 기록: 5년 보관
대금 결제 및 재화 등의 공급에 관한 기록: 5년 보관
소비자의 불만 또는 분쟁 처리에 관한 기록: 3년 보관

4. 개인정보의 파기 절차 및 방법
파기 절차
회원 탈퇴 요청 또는 개인정보 수집 및 이용 목적이 달성된 후 지체 없이 해당 정보를 파기합니다.
파기 방법
전자적 파일 형태: 복구 및 재생이 불가능한 방법으로 영구 삭제
종이 문서 형태: 분쇄하거나 소각

5. 이용자의 권리 및 행사 방법
이용자는 언제든지 자신의 개인정보에 대해 열람, 수정, 삭제, 처리 정지를 요구할 수 있습니다.
회원 탈퇴를 원하시는 경우, 서비스 내의 "회원 탈퇴" 기능을 이용하시거나 고객센터를 통해 요청하실 수 있습니다.

6. 동의를 거부할 권리 및 거부 시 불이익
이용자는 개인정보 수집 및 이용에 대한 동의를 거부할 권리가 있습니다.
그러나 필수 항목에 대한 동의를 거부하실 경우 서비스 이용이 제한될 수 있습니다.

7. 개인정보 보호책임자
연락처: eld_yeojh@naver.com

8. 개인정보의 안전성 확보 조치
회사는 개인정보의 안전한 처리를 위하여 기술적, 관리적 보호조치를 시행하고 있습니다.
개인정보의 암호화
해킹 등에 대비한 대책
접근 통제 장치의 설치 및 운영
''',
                  style: TextStyle(fontSize: 14),
                ),
              ],
            ),
          ),
          actions: [
            TextButton(
              style: TextButton.styleFrom(
                backgroundColor: Colors.black,
                foregroundColor: Colors.white,
              ),
              onPressed: () => Navigator.pop(ctx, false),
              child: Text('거부'),
            ),
            TextButton(
              style: TextButton.styleFrom(
                backgroundColor: Colors.black,
                foregroundColor: Colors.white,
              ),
              onPressed: () => Navigator.pop(ctx, true),
              child: Text('동의'),
            ),
          ],
        );
      },
    );
  }

  // ─────────────────────────────────────────
  // (F) 간단 Alert
  // ─────────────────────────────────────────
  void _showAlert(String title, String message) {
    showDialog(
      context: context,
      builder: (_) => AlertDialog(
        backgroundColor: Colors.white,
        title: Text(title, style: const TextStyle(color: Colors.black)),
        content: Text(message, style: const TextStyle(color: Colors.black)),
        actions: [
          TextButton(
            style: TextButton.styleFrom(
              backgroundColor: Colors.black,
              foregroundColor: Colors.white,
            ),
            onPressed: () => Navigator.pop(context),
            child: const Text('확인'),
          ),
        ],
      ),
    );
  }

  // ─────────────────────────────────────────
  // (G) 화면
  // ─────────────────────────────────────────
  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: _onWillPop,
      child: Stack(
        children: [
Scaffold(
      backgroundColor: Colors.white,

      // 상단 AppBar
      appBar: AppBar(
        title: const Text('ALL SCORE', style: TextStyle(color: Colors.white)),
        backgroundColor: Colors.black,
      ),

      // 전체 세로 레이아웃
      body: Column(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          // (1) 중앙 영역 → 로그인 UI
          Expanded(
            child: Center(
              child: SingleChildScrollView(
                padding: const EdgeInsets.all(16),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    // ─────────────────────────────────────────
                    // 1. 올스코어 로그인 (ID/PW)
                    // ─────────────────────────────────────────
                    const Text(
                      '올스코어 로그인',
                      style: TextStyle(fontSize: 24, fontWeight: FontWeight.bold, color: Colors.black),
                    ),
                    const SizedBox(height: 16),

                    // (A) 아이디 입력
                    SizedBox(
                      width: 300,
                      child: TextField(
                        controller: idController,
                        decoration: InputDecoration(
                          labelText: 'ID',
                          border: OutlineInputBorder(
                            borderSide: const BorderSide(color: Colors.black),
                            borderRadius: BorderRadius.circular(8),
                          ),
                          enabledBorder: OutlineInputBorder(
                            borderSide: const BorderSide(color: Colors.black),
                            borderRadius: BorderRadius.circular(8),
                          ),
                        ),
                      ),
                    ),
                    const SizedBox(height: 12),

                    // (B) 비밀번호 입력
                    SizedBox(
                      width: 300,
                      child: TextField(
                        controller: passwordController,
                        obscureText: true,
                        decoration: InputDecoration(
                          labelText: 'PW',
                          border: OutlineInputBorder(
                            borderSide: const BorderSide(color: Colors.black),
                            borderRadius: BorderRadius.circular(8),
                          ),
                          enabledBorder: OutlineInputBorder(
                            borderSide: const BorderSide(color: Colors.black),
                            borderRadius: BorderRadius.circular(8),
                          ),
                        ),
                      ),
                    ),

                    // (C) 로그인 에러
                    if (loginErrorMessage.isNotEmpty)
                      Padding(
                        padding: const EdgeInsets.only(top: 8.0),
                        child: Text(
                          loginErrorMessage,
                          style: const TextStyle(color: Colors.red),
                        ),
                      ),

                    const SizedBox(height: 8),

                    // (D) 자동로그인 체크박스
                    SizedBox(
                      width: 300,
                      child: Row(
                        children: [
                          Checkbox(
                            value: autoLogin,
                            onChanged: (val) {
                              setState(() {
                                autoLogin = val ?? false;
                              });
                            },
                          ),
                          const Text('자동로그인', style: TextStyle(color: Colors.black)),
                        ],
                      ),
                    ),

                    // (E) 로그인 버튼
                    SizedBox(
                      width: 300,
                      child: ElevatedButton(
                        style: ElevatedButton.styleFrom(
                          backgroundColor: Colors.white,
                          foregroundColor: Colors.black,
                          side: const BorderSide(color: Colors.black),
                          shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                        ),
                        onPressed: _loginWithIdPw,
                        child: const Text('로그인'),
                      ),
                    ),

                    // (F) ID/PW 찾기, 회원가입
                    const SizedBox(height: 8),
                    TextButton(
                      onPressed: () {
                        Navigator.push(context, MaterialPageRoute(builder: (_) => const IdFindingPage()));
                      },
                      child: const Text('ID 찾기', style: TextStyle(color: Colors.black)),
                    ),
                    TextButton(
                      onPressed: () {
                        Navigator.push(context, MaterialPageRoute(builder: (_) => const PwFindingPage()));
                      },
                      child: const Text('PW 찾기', style: TextStyle(color: Colors.black)),
                    ),
                    TextButton(
                      onPressed: () {
                        Navigator.push(context, MaterialPageRoute(builder: (_) => const SignUpPage()));
                      },
                      child: const Text('회원가입', style: TextStyle(color: Colors.black)),
                    ),

                    const SizedBox(height: 24),
                    const Divider(height: 1, color: Colors.black),
                    const SizedBox(height: 24),

                    // ─────────────────────────────────────────
                    // 2. 구글 로그인 / 회원가입
                    // ─────────────────────────────────────────
                    const Text(
                      '구글 계정',
                      style: TextStyle(fontSize: 20, color: Colors.black, fontWeight: FontWeight.bold),
                    ),
                    const SizedBox(height: 16),

                    // (a) 구글 로그인
                    SizedBox(
                      width: 300,
                      child: ElevatedButton.icon(
                        icon: Container(
                          width: 24,
                          height: 24,
                          decoration: const BoxDecoration(
                            image: DecorationImage(
                              image: AssetImage('assets/images/icons8-google-logo-48.png'),
                              fit: BoxFit.contain,
                            ),
                          ),
                        ),
                        label: const Text(
                          'Google 로그인',
                          style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold),
                        ),
                        style: ElevatedButton.styleFrom(
                          backgroundColor: Colors.white,
                          foregroundColor: Colors.black,
                          side: const BorderSide(color: Colors.black),
                          padding: const EdgeInsets.symmetric(vertical: 12, horizontal: 24),
                          shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                        ),
                        onPressed: _googleLogin,
                      ),
                    ),
                    const SizedBox(height: 12),

                    // (b) 구글 회원가입
                    SizedBox(
                      width: 300,
                      child: ElevatedButton.icon(
                        icon: Container(
                          width: 24,
                          height: 24,
                          decoration: const BoxDecoration(
                            image: DecorationImage(
                              image: AssetImage('assets/images/icons8-google-logo-48.png'),
                              fit: BoxFit.contain,
                            ),
                          ),
                        ),
                        label: const Text(
                          'Google 회원가입',
                          style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold),
                        ),
                        style: ElevatedButton.styleFrom(
                          backgroundColor: Colors.white,
                          foregroundColor: Colors.black,
                          side: const BorderSide(color: Colors.black),
                          padding: const EdgeInsets.symmetric(vertical: 12, horizontal: 24),
                          shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                        ),
                        onPressed: _googleSignUp,
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),

          // (2) 하단 광고 영역
          if (_isBannerReady && _bannerAd != null)
            Container(
              width: _bannerAd!.size.width.toDouble(),
              height: _bannerAd!.size.height.toDouble(),
              alignment: Alignment.center,
              child: AdWidget(ad: _bannerAd!),
            )
          else
            Container(
              width: 300,
              height: 50,
              color: Colors.grey.shade400,
              alignment: Alignment.center,
              child: const Text(
                '광고 로딩중',
                style: TextStyle(color: Colors.black),
              ),
            ),
        ],
        ),
      ),

        // (2) 로딩 중일 때 오버레이 표시
        if (_isLoading)
          Container(
            width: double.infinity,
            height: double.infinity,
            color: Colors.black54,    // 반투명 배경
            alignment: Alignment.center,
            child: const CircularProgressIndicator(color: Colors.white),
          ),
        ]
      )
    );
  }
}
