import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../../plugins/api.dart';
import '../../dialogs/response_dialog.dart';
import '../../dialogs/room_setting_finish_dialog.dart';
import '../../dialogs/user_info_finish_dialog.dart';

class FinishPrivatePage extends StatefulWidget {
  final int roomSeq;
  final bool fromPlayingPage; // 만약 대기/진행중에서 넘어온 경우 => 뒤로가기 시 메인으로

  const FinishPrivatePage({
    Key? key,
    required this.roomSeq,
    this.fromPlayingPage = false,
  }) : super(key: key);

  @override
  State<FinishPrivatePage> createState() => _FinishPrivatePageState();
}

class _FinishPrivatePageState extends State<FinishPrivatePage> {
  bool _isLoading = true;

  Map<String, dynamic> _roomInfo = {}; // 서버에서 받은 room_info
  // 전체 user_info Map
  //   userSeq → { user_seq, nickname, participant_type, score, ... }
  Map<String, dynamic> _userMap = {};

  // 리스트로 만든 (관리자 제외) 참가자 목록 (점수 내림차순)
  List<Map<String, dynamic>> _playerList = [];
  // 별도 사회자(ADMIN) 목록 (개인전이라 1명이거나 없을 수 있음)
  List<Map<String, dynamic>> _adminList = [];

  String _roomTitle = '';
  DateTime? _startDt;
  DateTime? _endDt;
  int _masterUserSeq = 0; // 방장 user_seq (ADMIN과는 별개)

  @override
  void initState() {
    super.initState();
    _fetchFinishRoomInfo();
  }

  /// (A) 서버에서 종료된 방 정보를 가져옴
  Future<void> _fetchFinishRoomInfo() async {
    setState(() => _isLoading = true);
    try {
      final body = {"room_seq": "${widget.roomSeq}"};
      final response = await Api.serverRequest(
        uri: '/room/score/get/finish/room/info',
        body: body,
      );
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          final data = resp['data'] ?? {};

          final rInfo = data['room_info'] ?? {};
          final uInfo = data['user_info'] ?? {};

          final rTitle = (rInfo['room_title'] ?? '') as String;
          final mSeq   = (rInfo['master_user_seq'] ?? 0) as int;
          final sdt    = rInfo['start_dt'];
          final edt    = rInfo['end_dt'];

          setState(() {
            _roomInfo      = rInfo;
            _userMap       = uInfo;
            _roomTitle     = rTitle.isNotEmpty ? rTitle : '종료된 방(개인전)';
            _masterUserSeq = mSeq;

            if (sdt != null && sdt is String && sdt.contains('T')) {
              _startDt = DateTime.tryParse(sdt);
            }
            if (edt != null && edt is String && edt.contains('T')) {
              _endDt = DateTime.tryParse(edt);
            }
          });

          // userInfo -> List 변환
          final List<Map<String, dynamic>> tempList = [];
          uInfo.forEach((_, val) {
            // val: { user_seq, participant_type, nickname, score, ... }
            tempList.add(Map<String, dynamic>.from(val));
          });

          // (1) 사회자(ADMIN) 분리
          final adminList = tempList.where((u) {
            final pType = (u['participant_type'] ?? '').toString().toUpperCase();
            return pType == 'ADMIN';
          }).toList();

          // (2) 플레이어 목록 (ADMIN 제외) & 점수 내림차순
          final playerList = tempList.where((u) {
            final pType = (u['participant_type'] ?? '').toString().toUpperCase();
            return pType != 'ADMIN';
          }).toList();

          // 점수 내림차순 정렬
          playerList.sort((a, b) {
            final sa = (a['score'] ?? 0) as int;
            final sb = (b['score'] ?? 0) as int;
            return sb.compareTo(sa); // 내림차순
          });

          setState(() {
            _adminList   = adminList;
            _playerList  = playerList;
            _isLoading   = false;
          });
        } else {
          final msgTitle   = resp['response_info']?['msg_title'] ?? '오류';
          final msgContent = resp['response_info']?['msg_content'] ?? '데이터 조회 실패';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신 오류');
      }
    } catch (e) {
      showResponseDialog(context, '오류', '$e');
    } finally {
      setState(() => _isLoading = false);
    }
  }

  /// (B) 뒤로가기
  Future<bool> _onWillPop() async {
    if (widget.fromPlayingPage) {
      // 진행중/대기중 등에서 넘어왔다면 => 메인으로
      Navigator.popUntil(context, (route) => route.isFirst);
    } else {
      // 검색 등에서 왔으면 => 한 단계 pop
      Navigator.pop(context);
    }
    return false;
  }

  /// (C) 방 정보보기 모달 (읽기 전용)
  Future<void> _openRoomSettingDialog() async {
    if (_roomInfo.isEmpty) return;
    await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (_) => RoomSettingFinishDialog(roomInfo: _roomInfo),
    );
  }

  /// (D) 게임 진행 시간 표시
  Widget _buildGameTimeWidget() {
    if (_startDt == null || _endDt == null) {
      return const Text('게임 진행 시간: 00:00', style: TextStyle(fontSize: 14));
    }
    final dur = _endDt!.difference(_startDt!);
    final hh = dur.inHours.toString().padLeft(2, '0');
    final mm = dur.inMinutes.remainder(60).toString().padLeft(2, '0');
    return Text('게임 진행 시간: $hh:$mm', style: const TextStyle(fontSize: 14));
  }

  /// (E) 플레이어 목록 표시 (점수 내림차순)
  /// - 1등/2등/3등 금/은/동 메달
  Widget _buildPlayerItem(Map<String, dynamic> user, int index) {
    final score      = (user['score'] ?? 0) as int;
    final nickname   = user['nickname'] ?? '유저';
    final profileImg = user['profile_img'] ?? '';
    final userSeq    = user['user_seq'] ?? 0;

    Widget medal = const SizedBox();
    if (index == 0) {
      medal = const Text('🥇 ', style: TextStyle(fontSize: 16));
    } else if (index == 1) {
      medal = const Text('🥈 ', style: TextStyle(fontSize: 16));
    } else if (index == 2) {
      medal = const Text('🥉 ', style: TextStyle(fontSize: 16));
    }

    return GestureDetector(
      onTap: () => _onTapUser(user),
      child: Container(
        margin: const EdgeInsets.symmetric(vertical: 4),
        child: Row(
          children: [
            medal,
            // 프로필
            Container(
              width: 36, height: 36,
              decoration: BoxDecoration(
                shape: BoxShape.circle,
                border: Border.all(color: Colors.black54),
              ),
              child: ClipOval(
                child: (profileImg.isNotEmpty)
                    ? Image.network(
                        'https://eldsoft.com:8097/images$profileImg',
                        fit: BoxFit.cover,
                        errorBuilder: (_, __, ___) => const Center(child: Text('ERR')),
                      )
                    : const Center(child: Text('No\nImg', textAlign: TextAlign.center, style: TextStyle(fontSize: 10))),
              ),
            ),
            const SizedBox(width: 8),
            Expanded(
              child: Text(nickname, style: const TextStyle(fontSize: 14, fontWeight: FontWeight.bold)),
            ),
            Text('$score 점', style: const TextStyle(fontSize: 14)),
          ],
        ),
      ),
    );
  }

  /// 사회자 목록 (보통 1명 예상)
  Widget _buildAdminItem(Map<String, dynamic> admin) {
    final nickname   = admin['nickname'] ?? '사회자';
    final profileImg = admin['profile_img'] ?? '';
    return GestureDetector(
      onTap: () => _onTapUser(admin),
      child: Row(
        children: [
          Container(
            width: 36, height: 36,
            margin: const EdgeInsets.only(right: 8),
            decoration: BoxDecoration(
              shape: BoxShape.circle,
              border: Border.all(color: Colors.deepPurple),
            ),
            child: ClipOval(
              child: (profileImg.isNotEmpty)
                  ? Image.network(
                      'https://eldsoft.com:8097/images$profileImg',
                      fit: BoxFit.cover,
                      errorBuilder: (_, __, ___) => const Center(child: Text('ERR')),
                    )
                  : const Center(child: Text('No\nImg', textAlign: TextAlign.center, style: TextStyle(fontSize: 10))),
            ),
          ),
          Text(nickname, style: const TextStyle(fontSize: 14, fontWeight: FontWeight.bold, color: Colors.deepPurple)),
        ],
      ),
    );
  }

  /// 사용자 클릭 -> 새 유저 정보 모달
  Future<void> _onTapUser(Map<String, dynamic> userData) async {
    // user_info_finish_dialog.dart (새 모달)
    await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (_) => UserInfoFinishDialog(userData: userData),
    );
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: _onWillPop,
      child: Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          backgroundColor: Colors.black,
          leading: IconButton(
            icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
            onPressed: _onWillPop,
          ),
          title: Text(_roomTitle, style: const TextStyle(color: Colors.white)),
        ),
        body: _isLoading
            ? const Center(child: CircularProgressIndicator())
            : SingleChildScrollView(
                padding: const EdgeInsets.all(16),
                child: Column(
                  children: [
                    // (A) 상단: [방 정보 보기] 버튼 + 진행 시간
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        // 방번호는 숨기라 했으므로 제거
                        ElevatedButton(
                          onPressed: _openRoomSettingDialog,
                          style: ElevatedButton.styleFrom(
                            backgroundColor: Colors.white,
                            side: const BorderSide(color: Colors.black, width: 1),
                          ),
                          child: const Text('방 정보 보기', style: TextStyle(color: Colors.black)),
                        ),
                        _buildGameTimeWidget(),
                      ],
                    ),
                    const SizedBox(height: 16),

                    // (B) 사회자 목록
                    if (_adminList.isNotEmpty) ...[
                      Container(
                        width: double.infinity,
                        padding: const EdgeInsets.all(8),
                        decoration: BoxDecoration(
                          border: Border.all(color: Colors.deepPurple),
                          borderRadius: BorderRadius.circular(8),
                        ),
                        child: Row(
                          children: [
                            const Text('사회자: ', style: TextStyle(fontSize: 14, color: Colors.deepPurple)),
                            ..._adminList.map(_buildAdminItem),
                          ],
                        ),
                      ),
                      const SizedBox(height: 16),
                    ],

                    // (C) 참가자 목록
                    ListView.builder(
                      primary: false,
                      shrinkWrap: true,
                      itemCount: _playerList.length,
                      itemBuilder: (ctx, i) => _buildPlayerItem(_playerList[i], i),
                    ),
                  ],
                ),
              ),
      ),
    );
  }
}
