import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../../plugins/api.dart';
import '../../dialogs/response_dialog.dart';
import '../../dialogs/room_setting_finish_dialog.dart';
import '../../dialogs/user_info_finish_dialog.dart';

class FinishTeamPage extends StatefulWidget {
  final int roomSeq;
  final bool fromPlayingPage; // 진행중에서 넘어왔는지 여부

  const FinishTeamPage({
    Key? key,
    required this.roomSeq,
    this.fromPlayingPage = false,
  }) : super(key: key);

  @override
  State<FinishTeamPage> createState() => _FinishTeamPageState();
}

class _FinishTeamPageState extends State<FinishTeamPage> {
  bool _isLoading = true;

  Map<String, dynamic> _roomInfo = {};
  Map<String, dynamic> _userMap = {};
  List<Map<String, dynamic>> _userList = [];

  String _roomTitle = '';
  int _masterUserSeq = 0;
  DateTime? _startDt;
  DateTime? _endDt;

  // 팀별 [ { user }, { user } ... ]
  Map<String, List<Map<String, dynamic>>> _teamMap = {};
  // 팀별 점수
  Map<String, int> _teamScoreMap = {};

  // 별도 사회자 목록
  List<Map<String, dynamic>> _adminList = [];

  @override
  void initState() {
    super.initState();
    _fetchFinishRoomInfo();
  }

  Future<void> _fetchFinishRoomInfo() async {
    setState(() => _isLoading = true);
    try {
      final body = {"room_seq": "${widget.roomSeq}"};
      final response = await Api.serverRequest(
        uri: '/room/score/get/finish/room/info',
        body: body,
      );
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          final data  = resp['data'] ?? {};
          final rInfo = data['room_info'] ?? {};
          final uInfo = data['user_info'] ?? {};

          final rTitle = (rInfo['room_title'] ?? '') as String;
          final mSeq   = (rInfo['master_user_seq'] ?? 0) as int;
          final sdt    = rInfo['start_dt'];
          final edt    = rInfo['end_dt'];

          setState(() {
            _roomInfo      = rInfo;
            _userMap       = uInfo;
            _roomTitle     = rTitle.isNotEmpty ? rTitle : '종료된 팀전';
            _masterUserSeq = mSeq;

            if (sdt != null && sdt is String && sdt.contains('T')) {
              _startDt = DateTime.tryParse(sdt);
            }
            if (edt != null && edt is String && edt.contains('T')) {
              _endDt = DateTime.tryParse(edt);
            }
          });

          // userList
          final List<Map<String, dynamic>> tempList = [];
          uInfo.forEach((_, val) {
            tempList.add(Map<String, dynamic>.from(val));
          });

          // (1) 사회자(ADMIN) 분리
          final adminList = tempList.where((u) {
            final pType = (u['participant_type'] ?? '').toString().toUpperCase();
            return pType == 'ADMIN';
          }).toList();

          // (2) 일반 참가자
          final players = tempList.where((u) {
            final pType = (u['participant_type'] ?? '').toString().toUpperCase();
            return pType != 'ADMIN';
          }).toList();

          // (3) 팀명별 분류 + 점수 합
          final Map<String, List<Map<String, dynamic>>> tMap = {};
          final Map<String, int> tScoreMap = {};

          for (var user in players) {
            final tName = (user['team_name'] ?? 'WAIT').toString().toUpperCase();
            if (tName == 'WAIT') continue; // 팀 미배정
            tMap.putIfAbsent(tName, () => []);
            tMap[tName]!.add(user);
          }

          // 팀별 점수
          tMap.forEach((team, mems) {
            int sumScore = 0;
            // mems 내림차순 정렬
            mems.sort((a, b) {
              final sa = (a['score'] ?? 0) as int;
              final sb = (b['score'] ?? 0) as int;
              return sb.compareTo(sa);
            });
            for (var m in mems) {
              sumScore += (m['score'] ?? 0) as int;
            }
            tScoreMap[team] = sumScore;
          });

          // (4) 팀들을 점수 순으로 정렬
          final sortedTeams = tScoreMap.keys.toList();
          sortedTeams.sort((a, b) => tScoreMap[b]!.compareTo(tScoreMap[a]!));

          // 정렬된 결과를 새 맵에 담음
          final Map<String, List<Map<String, dynamic>>> finalTeamMap = {};
          final Map<String, int> finalScoreMap = {};

          for (var t in sortedTeams) {
            finalTeamMap[t] = tMap[t]!;
            finalScoreMap[t] = tScoreMap[t]!;
          }

          setState(() {
            _adminList     = adminList;
            _userList      = tempList; // 전체 필요하면 보관
            _teamMap       = finalTeamMap;
            _teamScoreMap  = finalScoreMap;
            _isLoading     = false;
          });
        } else {
          final msgTitle   = resp['response_info']?['msg_title'] ?? '오류';
          final msgContent = resp['response_info']?['msg_content'] ?? '데이터 조회 실패';
          showResponseDialog(context, msgTitle, msgContent);
        }
      } else {
        showResponseDialog(context, '실패', '서버 통신 오류');
      }
    } catch (e) {
      showResponseDialog(context, '오류', '$e');
    } finally {
      setState(() => _isLoading = false);
    }
  }

  Future<bool> _onWillPop() async {
    if (widget.fromPlayingPage) {
      Navigator.popUntil(context, (route) => route.isFirst);
    } else {
      Navigator.pop(context);
    }
    return false;
  }

  Future<void> _openRoomSettingDialog() async {
    if (_roomInfo.isEmpty) return;
    await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (_) => RoomSettingFinishDialog(roomInfo: _roomInfo),
    );
  }

  Widget _buildGameTimeWidget() {
    if (_startDt == null || _endDt == null) {
      return const Text('게임 진행 시간: 00:00', style: TextStyle(fontSize: 14));
    }
    final dur = _endDt!.difference(_startDt!);
    final hh = dur.inHours.toString().padLeft(2, '0');
    final mm = dur.inMinutes.remainder(60).toString().padLeft(2, '0');
    return Text('게임 진행 시간: $hh:$mm', style: const TextStyle(fontSize: 14));
  }

  /// 팀 박스 (팀별 점수 표시 + 1등2등3등 금/은/동 배경)
  Widget _buildTeamBox(String teamName, int index) {
    final members = _teamMap[teamName] ?? [];
    final tScore  = _teamScoreMap[teamName] ?? 0;

    // 1등/2등/3등 팀 -> 배경색
    Color bgColor = Colors.white;
    if (index == 0) {
      bgColor = const Color(0xFFFFF9C4); // 약간 금색 계열 예: amber.shade100
    } else if (index == 1) {
      bgColor = const Color(0xFFE0E0E0); // 은색(회색) 계열
    } else if (index == 2) {
      bgColor = const Color(0xFFFFE0B2); // 동색(주황~갈색 계열)
    }

    return Container(
      margin: const EdgeInsets.symmetric(vertical: 8),
      decoration: BoxDecoration(
        color: bgColor,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: Column(
        children: [
          // 상단 바
          Container(
            color: Colors.black,
            width: double.infinity,
            padding: const EdgeInsets.all(8),
            child: Center(
              child: Text(
                '$teamName 팀 (점수: $tScore)',
                style: const TextStyle(color: Colors.white, fontWeight: FontWeight.bold),
              ),
            ),
          ),
          // 팀 멤버
          Container(
            padding: const EdgeInsets.all(8),
            child: SingleChildScrollView(
              scrollDirection: Axis.horizontal,
              child: Row(
                children: members.map((m) => _buildTeamMember(m)).toList(),
              ),
            ),
          ),
        ],
      ),
    );
  }

  /// 팀 멤버 표시
  Widget _buildTeamMember(Map<String, dynamic> user) {
    final score      = (user['score'] ?? 0) as int;
    final nickname   = user['nickname'] ?? '유저';
    final profileImg = user['profile_img'] ?? '';

    return GestureDetector(
      onTap: () => _onTapUser(user),
      child: Container(
        width: 60,
        margin: const EdgeInsets.only(right: 8),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Text('$score', style: const TextStyle(fontSize: 16, fontWeight: FontWeight.bold)),
            const SizedBox(height: 2),
            Container(
              width: 30, height: 30,
              decoration: BoxDecoration(
                shape: BoxShape.circle,
                border: Border.all(color: Colors.black),
              ),
              child: ClipOval(
                child: Image.network(
                  'https://eldsoft.com:8097/images$profileImg',
                  fit: BoxFit.cover,
                  errorBuilder: (ctx, err, st) => const Center(
                    child: Text('ERR', style: TextStyle(fontSize: 8)),
                  ),
                ),
              ),
            ),
            const SizedBox(height: 2),
            Text(nickname, style: const TextStyle(fontSize: 11), overflow: TextOverflow.ellipsis),
          ],
        ),
      ),
    );
  }

  /// 팀전 사회자 목록 (ADMIN)
  Widget _buildAdminList() {
    final adminList = _adminList;
    if (adminList.isEmpty) return const SizedBox();

    return Container(
      width: double.infinity,
      margin: const EdgeInsets.only(top: 16),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.deepPurple),
        borderRadius: BorderRadius.circular(8),
      ),
      child: Column(
        children: [
          Container(
            color: Colors.black,
            width: double.infinity,
            padding: const EdgeInsets.all(8),
            child: const Center(
              child: Text('사회자', style: TextStyle(color: Colors.white, fontWeight: FontWeight.bold)),
            ),
          ),
          ListView.builder(
            shrinkWrap: true,
            primary: false,
            itemCount: adminList.length,
            itemBuilder: (ctx, i) {
              final user = adminList[i];
              return _buildAdminItem(user);
            },
          ),
        ],
      ),
    );
  }

  Widget _buildAdminItem(Map<String, dynamic> user) {
    final nickname   = user['nickname']   ?? '사회자';
    final profileImg = user['profile_img'] ?? '';

    return ListTile(
      onTap: () => _onTapUser(user),
      leading: CircleAvatar(
        backgroundColor: Colors.white,
        backgroundImage: (profileImg.isNotEmpty)
            ? NetworkImage('https://eldsoft.com:8097/images$profileImg')
            : null,
        child: (profileImg.isEmpty)
            ? const Text('NoImg', style: TextStyle(fontSize: 10))
            : null,
      ),
      title: Text(nickname),
      subtitle: const Text('사회자'),
    );
  }

  Future<void> _onTapUser(Map<String, dynamic> userData) async {
    // 새 유저 정보 모달
    await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (_) => UserInfoFinishDialog(userData: userData),
    );
  }

  @override
  Widget build(BuildContext context) {
    final teamNames = _teamMap.keys.toList();
    // 이미 점수순으로 정렬됨

    return WillPopScope(
      onWillPop: _onWillPop,
      child: Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          backgroundColor: Colors.black,
          leading: IconButton(
            icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
            onPressed: _onWillPop,
          ),
          title: Text(_roomTitle, style: const TextStyle(color: Colors.white)),
        ),
        body: _isLoading
            ? const Center(child: CircularProgressIndicator())
            : SingleChildScrollView(
                padding: const EdgeInsets.all(16),
                child: Column(
                  children: [
                    // 방 정보 + 시간
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        ElevatedButton(
                          onPressed: _openRoomSettingDialog,
                          style: ElevatedButton.styleFrom(
                            backgroundColor: Colors.white,
                            side: const BorderSide(color: Colors.black),
                          ),
                          child: const Text('방 정보 보기', style: TextStyle(color: Colors.black)),
                        ),
                        _buildGameTimeWidget(),
                      ],
                    ),
                    const SizedBox(height: 16),

                    // 팀 박스들
                    for (int i = 0; i < teamNames.length; i++)
                      _buildTeamBox(teamNames[i], i),

                    // 사회자 목록
                    _buildAdminList(),
                  ],
                ),
              ),
      ),
    );
  }
}
