import 'package:flutter/material.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'main_page.dart';
import 'finish_private_page.dart'; // 게임 종료 후 이동할 페이지
import '../../plugins/api.dart';
import '../../dialogs/response_dialog.dart';
import '../../dialogs/score_edit_dialog.dart';   // 점수 수정 모달
import '../../dialogs/user_info_basic_dialog.dart'; // 일반 유저 정보 모달

class PlayingPrivatePage extends StatefulWidget {
  final int roomSeq;
  final String roomTitle;

  const PlayingPrivatePage({
    Key? key,
    required this.roomSeq,
    required this.roomTitle,
  }) : super(key: key);

  @override
  State<PlayingPrivatePage> createState() => _PlayingPrivatePageState();
}

class _PlayingPrivatePageState extends State<PlayingPrivatePage> {
  late DatabaseReference _roomRef;
  Stream<DatabaseEvent>? _roomStream;

  String roomMasterYn = 'N';
  String roomTitle = '';

  int myScore = 0;
  List<Map<String, dynamic>> _scoreList = [];

  bool _isLoading = true;
  String mySeq = '0';

  // userListMap: { userSeq: true/false }
  Map<String, bool> _userListMap = {};

  @override
  void initState() {
    super.initState();
    // (1) FRD 연결 복원
    FirebaseDatabase.instance.goOnline();

    roomTitle = widget.roomTitle;
    _initFirebase();
  }

  Future<void> _initFirebase() async {
    final prefs = await SharedPreferences.getInstance();
    mySeq = prefs.getInt('my_user_seq')?.toString() ?? '0';

    final roomKey = 'korea-${widget.roomSeq}';
    _roomRef = FirebaseDatabase.instance.ref('rooms/$roomKey');

    _listenRoomData();
  }

  void _listenRoomData() {
    _roomStream = _roomRef.onValue;
    _roomStream?.listen((event) {
      final snapshot = event.snapshot;
      if (!snapshot.exists) {
        setState(() {
          _isLoading = false;
          roomTitle = '방 정보 없음';
          _scoreList = [];
          myScore = 0;
        });
        return;
      }

      final data = snapshot.value as Map<dynamic, dynamic>? ?? {};
      final roomInfoData = data['roomInfo'] as Map<dynamic, dynamic>? ?? {};
      final userInfoData = data['userInfo'] as Map<dynamic, dynamic>? ?? {};
      final userListData = data['userList'] as Map<dynamic, dynamic>?;

      // 방 상태 체크
      final roomStatus = (roomInfoData['room_status'] ?? 'WAIT').toString().toUpperCase();
      if (roomStatus == 'FINISH') {
        // 종료 페이지
        if (mounted) {
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (_) => FinishPrivatePage(roomSeq: widget.roomSeq)),
          );
        }
        return;
      }

      setState(() {
        // 방장 여부
        final masterSeq = roomInfoData['master_user_seq'];
        roomMasterYn = (masterSeq != null && masterSeq.toString() == mySeq) ? 'Y' : 'N';

        // 방 제목
        final newTitle = (roomInfoData['room_title'] ?? '') as String;
        if (newTitle.isNotEmpty) roomTitle = newTitle;

        // userListMap
        _userListMap.clear();
        if (userListData != null) {
          userListData.forEach((k, v) {
            _userListMap[k.toString()] = (v == true);
          });
        }

        // 전체 유저 목록
        final List<Map<String, dynamic>> rawList = [];
        userInfoData.forEach((uSeq, uData) {
          rawList.add({
            'user_seq': uSeq,
            'participant_type': (uData['participant_type'] ?? '').toString().toUpperCase(),
            'nickname': uData['nickname'] ?? '유저',
            'score': uData['score'] ?? 0,
            'profile_img': uData['profile_img'] ?? '',
            'department': uData['department'] ?? '',
            'introduce_myself': uData['introduce_myself'] ?? '',
            'is_my_score': (uSeq.toString() == mySeq) ? 'Y' : 'N',
          });
        });

        // 내 점수
        int tmpMyScore = 0;
        for (var user in rawList) {
          if ((user['is_my_score'] ?? 'N') == 'Y') {
            tmpMyScore = user['score'] ?? 0;
          }
        }

        // ADMIN 제외
        final playerList = rawList.where((u) => u['participant_type'] != 'ADMIN').toList();
        // 점수 내림차순
        playerList.sort((a, b) {
          final scoreA = a['score'] ?? 0;
          final scoreB = b['score'] ?? 0;
          return scoreB.compareTo(scoreA);
        });

        myScore = tmpMyScore;
        _scoreList = playerList;

        _isLoading = false;
      });
    }, onError: (err) {
      setState(() {
        _isLoading = false;
        roomTitle = '오류 발생';
      });
    });
  }

  /// 방장이면 Finish API
  Future<void> _requestFinish() async {
    final reqBody = {
      "room_seq": "${widget.roomSeq}",
      "room_type": "PRIVATE",
    };
    try {
      await Api.serverRequest(uri: '/room/score/game/finish', body: reqBody);
    } catch (e) {
      // ignore
    }
  }

  /// 뒤로가기
  Future<bool> _onWillPop() async {
    if (roomMasterYn == 'Y') {
      // 방장 -> 모달
      final confirm = await showDialog<bool>(
        context: context,
        barrierDismissible: false,
        builder: (_) {
          return AlertDialog(
            backgroundColor: Colors.white,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(16),
              side: const BorderSide(color: Colors.black, width: 1),
            ),
            title: const Text('나가기', style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            content: const Text(
              '방장이 나가면 게임이 종료됩니다.\n정말 나가시겠습니까?',
              style: TextStyle(fontSize: 14),
            ),
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context, false),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                ),
                child: const Text('취소'),
              ),
              TextButton(
                onPressed: () => Navigator.pop(context, true),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                ),
                child: const Text('종료'),
              ),
            ],
          );
        },
      );

      if (confirm != true) return false;

      // Finish API
      await _requestFinish();
    } else {
      // 일반 유저
      final confirm = await showDialog<bool>(
        context: context,
        barrierDismissible: false,
        builder: (_) {
          return AlertDialog(
            backgroundColor: Colors.white,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(16),
              side: const BorderSide(color: Colors.black, width: 1),
            ),
            title: const Text('나가기', style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            content: const Text(
              '진행중인 방에서 나가면 재입장이 불가능합니다.\n정말 나가시겠습니까?',
              style: TextStyle(fontSize: 14),
            ),
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context, false),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                ),
                child: const Text('취소'),
              ),
              TextButton(
                onPressed: () => Navigator.pop(context, true),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                ),
                child: const Text('나가기'),
              ),
            ],
          );
        },
      );

      if (confirm != true) return false;
    }

    // userList => false
    final userRef = _roomRef.child('userList').child(mySeq);
    await userRef.set(false);

    if (!mounted) return false;
    Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
    return false;
  }

  /// 참가자 카드
  Widget _buildScoreItem(Map<String, dynamic> user) {
    final userSeq = user['user_seq'].toString();
    final score   = user['score'] ?? 0;
    final nickname = user['nickname'] ?? '유저';

    final bool isActive = _userListMap[userSeq] ?? true;
    final hasExited = !isActive;

    return GestureDetector(
      onTap: () => _onTapUser(user),
      child: Container(
        width: 60,
        margin: const EdgeInsets.all(4),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            hasExited
                ? Text('X', style: const TextStyle(fontSize: 20, color: Colors.redAccent, fontWeight: FontWeight.bold))
                : Text('$score', style: const TextStyle(fontSize: 20, fontWeight: FontWeight.bold)),
            const SizedBox(height: 2),
            Container(
              width: 30, height: 30,
              decoration: BoxDecoration(
                shape: BoxShape.circle,
                border: Border.all(color: hasExited ? Colors.redAccent : Colors.black),
              ),
              child: hasExited
                  ? const Center(
                      child: Text('X', style: TextStyle(fontSize: 14, color: Colors.redAccent, fontWeight: FontWeight.bold)),
                    )
                  : ClipOval(
                      child: Image.network(
                        'https://eldsoft.com:8097/images${user['profile_img']}',
                        fit: BoxFit.cover,
                        errorBuilder: (_, __, ___) => const Center(child: Text('ERR', style: TextStyle(fontSize: 8))),
                      ),
                    ),
            ),
            const SizedBox(height: 2),
            Text(nickname,
                style: TextStyle(fontSize: 11, color: hasExited ? Colors.redAccent : Colors.black),
                overflow: TextOverflow.ellipsis),
          ],
        ),
      ),
    );
  }

  Future<void> _onTapUser(Map<String, dynamic> userData) async {
    final pType = (userData['participant_type'] ?? '').toString().toUpperCase();
    if (pType == 'ADMIN') {
      // 점수수정
      await showDialog(
        context: context,
        builder: (_) => ScoreEditDialog(
          roomSeq: widget.roomSeq,
          roomType: 'PRIVATE',
          userData: userData,
        ),
      );
    } else if (roomMasterYn == 'Y') {
      // 방장(PLAYER)도 수정 가능
      await showDialog(
        context: context,
        builder: (_) => ScoreEditDialog(
          roomSeq: widget.roomSeq,
          roomType: 'PRIVATE',
          userData: userData,
        ),
      );
    } else {
      // 일반 유저 정보
      await showDialog(
        context: context,
        builder: (_) => UserInfoBasicDialog(userData: userData),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: _onWillPop,
      child: Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          backgroundColor: Colors.black,
          elevation: 0,
          leading: IconButton(
            icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
            onPressed: () => _onWillPop(),
          ),
          title: Text(
            roomTitle.isNotEmpty ? roomTitle : '진행중 (개인전)',
            style: const TextStyle(color: Colors.white),
          ),
          actions: [
            if (roomMasterYn == 'Y')
              TextButton(
                onPressed: _requestFinish,
                child: const Text('게임종료', style: TextStyle(color: Colors.white)),
              ),
          ],
        ),
        body: _isLoading
            ? const Center(child: CircularProgressIndicator())
            : Column(
                children: [
                  // 내 점수
                  Container(
                    width: double.infinity,
                    padding: const EdgeInsets.symmetric(vertical: 16),
                    child: Column(
                      children: [
                        const Text('내 점수', style: TextStyle(fontSize: 14, fontWeight: FontWeight.bold)),
                        const SizedBox(height: 4),
                        Text('$myScore', style: const TextStyle(fontSize: 36, fontWeight: FontWeight.bold)),
                      ],
                    ),
                  ),
                  const Divider(height: 1, color: Colors.black),
                  Expanded(
                    child: SingleChildScrollView(
                      padding: const EdgeInsets.all(8),
                      child: Wrap(
                        spacing: 8,
                        runSpacing: 8,
                        children: _scoreList.map(_buildScoreItem).toList(),
                      ),
                    ),
                  ),
                  Container(
                    height: 50,
                    decoration: BoxDecoration(
                      color: Colors.white,
                      border: Border.all(color: Colors.black, width: 1),
                    ),
                    child: const Center(
                      child: Text('구글 광고'),
                    ),
                  ),
                ],
              ),
      ),
    );
  }
}
