import 'package:flutter/material.dart';
import 'dart:async';
import 'package:firebase_database/firebase_database.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'main_page.dart';
import '../../plugins/api.dart';
import '../../dialogs/response_dialog.dart';
import '../../dialogs/yes_no_dialog.dart';
import '../../dialogs/room_setting_dialog.dart';
import '../../dialogs/user_info_private_dialog.dart';
import 'playing_private_page.dart';

class WaitingRoomPrivatePage extends StatefulWidget {
  final int roomSeq;
  final String roomType; // "private"

  const WaitingRoomPrivatePage({
    Key? key,
    required this.roomSeq,
    required this.roomType,
  }) : super(key: key);

  @override
  State<WaitingRoomPrivatePage> createState() => _WaitingRoomPrivatePageState();
}

class _WaitingRoomPrivatePageState extends State<WaitingRoomPrivatePage> {
  // ─────────────────────────────────────────
  // 방 설정
  // ─────────────────────────────────────────
  String roomMasterYn = 'N';
  String roomTitle = '';
  String roomIntro = '';
  String openYn = 'Y';
  String roomPw = '';
  int runningTime = 1;
  int numberOfPeople = 10;
  String scoreOpenRange = 'PRIVATE';

  // FRD
  late DatabaseReference _roomRef;
  Stream<DatabaseEvent>? _roomStream;
  StreamSubscription<DatabaseEvent>? _roomStreamSubscription;

  // 유저 목록
  List<Map<String, dynamic>> _userList = [];

  bool _isLoading = true;

  // 진행중 화면 이동 중복 방지
  bool _movedToRunningPage = false;
  // 강퇴 안내 중복 방지
  bool _kickedOut = false;

  // 내 user_seq
  String mySeq = '0';

  // ─────────────────────────────────────────
  // 1시간 카운트다운
  // ─────────────────────────────────────────
  Timer? _countdownTimer;
  Duration _remaining = const Duration(hours: 1); // 기본 1시간
  DateTime? _createDt; // FRD의 roomInfo.create_dt

  @override
  void initState() {
    super.initState();
    // FRD 연결 복원
    FirebaseDatabase.instance.goOnline();
    _initRoomRef();
  }

  Future<void> _initRoomRef() async {
    final prefs = await SharedPreferences.getInstance();
    mySeq = prefs.getInt('my_user_seq')?.toString() ?? '0';

    final roomKey = 'korea-${widget.roomSeq}';
    _roomRef = FirebaseDatabase.instance.ref('rooms/$roomKey');

    // onDisconnect + connect_yn='Y'
    final myUserRef = _roomRef.child('userInfo').child(mySeq);
    myUserRef.onDisconnect().update({'connect_yn': 'N'});
    await myUserRef.update({'connect_yn': 'Y'});

    _listenRoomData();
  }

  void _listenRoomData() {
    _roomStream = _roomRef.onValue;
    _roomStreamSubscription = _roomStream?.listen((event) {
      final snapshot = event.snapshot;
      if (!snapshot.exists) {
        setState(() {
          _isLoading = false;
          roomTitle = '방 정보 없음';
          _userList = [];
        });
        return;
      }

      final data = snapshot.value as Map<dynamic, dynamic>? ?? {};
      final roomInfoData = data['roomInfo'] as Map<dynamic, dynamic>? ?? {};
      final userInfoData = data['userInfo'] as Map<dynamic, dynamic>? ?? {};

      final roomStatus = (roomInfoData['room_status'] ?? 'WAIT').toString().toUpperCase();

      // (A) roomInfo 갱신
      setState(() {
        roomTitle = (roomInfoData['room_title'] ?? '') as String;
        roomIntro = (roomInfoData['room_intro'] ?? '') as String;
        openYn = (roomInfoData['open_yn'] ?? 'Y') as String;
        roomPw = (roomInfoData['room_pw'] ?? '') as String;
        runningTime = _toInt(roomInfoData['running_time'], 1);
        numberOfPeople = _toInt(roomInfoData['number_of_people'], 10);
        scoreOpenRange = (roomInfoData['score_open_range'] ?? 'PRIVATE') as String;

        // 방장 여부
        roomMasterYn = 'N';
        final masterSeq = roomInfoData['master_user_seq'];
        if (masterSeq != null && masterSeq.toString() == mySeq) {
          roomMasterYn = 'Y';
        }

        // userList
        final tempList = <Map<String, dynamic>>[];
        userInfoData.forEach((userSeq, userMap) {
          tempList.add({
            'user_seq': userSeq,
            'participant_type': userMap['participant_type'] ?? '',
            'nickname': userMap['nickname'] ?? '유저',
            'score': userMap['score'] ?? 0,
            'profile_img': userMap['profile_img'] ?? '',
            'introduce_myself': userMap['introduce_myself'] ?? '',
            'ready_yn': (userMap['ready_yn'] ?? 'N').toString().toUpperCase(),
            'connect_yn': (userMap['connect_yn'] ?? 'Y').toString().toUpperCase(),
          });
        });
        _userList = tempList;
        _isLoading = false;
      });

      // (B) 진행중 => 이동
      if (roomStatus == 'RUNNING' && !_movedToRunningPage) {
        _movedToRunningPage = true;
        Navigator.pushReplacement(
          context,
          MaterialPageRoute(
            builder: (_) => PlayingPrivatePage(
              roomSeq: widget.roomSeq,
              roomTitle: roomTitle,
            ),
          ),
        );
        return;
      }

      // (C) 1시간 카운트다운 위한 create_dt 파싱
      // 예: "2025-01-07T06:38:10.123456"
      final createDtStr = (roomInfoData['create_dt'] ?? '') as String;
      if (createDtStr.isNotEmpty && createDtStr.contains('T')) {
        final dt = DateTime.tryParse(createDtStr);
        if (dt != null) {
          _createDt = dt; 
          _startCountdownTimer(); 
        }
      }

      // (D) 내가 목록에서 사라졌는지 => 강퇴 판별
      final amIStillHere = _userList.any((u) => u['user_seq'].toString() == mySeq);
      if (!amIStillHere && !_kickedOut && roomMasterYn != 'Y') {
        _kickedOut = true;
        Future.delayed(Duration.zero, () async {
          await showResponseDialog(context, '안내', '강퇴되었습니다.');
          Navigator.pushReplacement(
            context,
            MaterialPageRoute(builder: (_) => const MainPage()),
          );
        });
      }
    }, onError: (error) {
      setState(() {
        _isLoading = false;
        roomTitle = '오류 발생';
      });
    });
  }

  // 1시간 카운트다운 타이머
  void _startCountdownTimer() {
    if (_countdownTimer != null && _countdownTimer!.isActive) {
      return; // 이미 실행중이면 중복 실행 방지
    }
    if (_createDt == null) return;

    _countdownTimer = Timer.periodic(const Duration(seconds: 1), (timer) {
      // 목표시각: createDt + 1시간
      final endTime = _createDt!.add(const Duration(hours: 1));
      final now = DateTime.now();
      final diff = endTime.difference(now);

      if (diff.isNegative) {
        // 이미 시간이 지남 -> 자동 종료 로직
        timer.cancel();
        _remaining = const Duration(seconds: 0);
        _onAutoTimeout(); 
      } else {
        setState(() {
          _remaining = diff;
        });
      }
    });
  }

  // 1시간 만료 후 자동 종료
  void _onAutoTimeout() {
    // 방장 => 방 삭제 (leave API)
    // 일반 => 그냥 나가기
    if (roomMasterYn == 'Y') {
      _requestLeaveRoom(); 
    } else {
      _requestLeaveRoom(); 
    }
  }

  Future<void> _requestLeaveRoom() async {
    try {
      final reqBody = {"room_seq": "${widget.roomSeq}"};
      final response = await Api.serverRequest(uri: '/room/score/game/leave', body: reqBody);
      // result ok -> 메인
    } catch (e) {
      //
    }
    if (mounted) {
      Navigator.pushReplacement(context, MaterialPageRoute(builder: (_) => const MainPage()));
    }
  }

  @override
  void dispose() {
    _countdownTimer?.cancel();
    _roomStreamSubscription?.cancel();
    super.dispose();
  }

  /// 뒤로가기 → 방 나가기
  Future<void> _onLeaveRoom() async {
    if (roomMasterYn == 'Y') {
      // 방장
      final confirm = await showDialog<bool>(
        context: context,
        barrierDismissible: false,
        builder: (_) {
          return AlertDialog(
            backgroundColor: Colors.white,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(16),
              side: const BorderSide(color: Colors.black, width: 2),
            ),
            title: const Center(
              child: Text('방 나가기', style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold)),
            ),
            content: const Text('방장이 나가면 방은 삭제됩니다.\n정말 나가시겠습니까?', style: TextStyle(fontSize: 14)),
            actionsAlignment: MainAxisAlignment.spaceEvenly,
            actions: [
              TextButton(
                onPressed: () => Navigator.pop(context, false),
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                  shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                ),
                child: const Text('취소'),
              ),
              TextButton(
                onPressed: () {
                  final myUserRef = _roomRef.child('userInfo').child(mySeq);
                  myUserRef.onDisconnect().cancel();
                  Navigator.pop(context, true);
                },
                style: TextButton.styleFrom(
                  backgroundColor: Colors.black,
                  foregroundColor: Colors.white,
                  shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8)),
                ),
                child: const Text('확인'),
              ),
            ],
          );
        },
      );
      if (confirm != true) return;

      // leave API
      await _requestLeaveRoom();
    } else {
      // 일반
      await _requestLeaveRoom();
    }
  }

  int _toInt(dynamic val, int defaultVal) {
    if (val == null) return defaultVal;
    if (val is int) return val;
    if (val is String) {
      return int.tryParse(val) ?? defaultVal;
    }
    return defaultVal;
  }

  /// 상단 버튼
  Widget _buildTopButtons() {
    if (_isLoading) return const SizedBox();

    final me = _userList.firstWhere((u) => (u['user_seq'] ?? '0') == mySeq, orElse: () => {});
    final myReadyYn = (me['ready_yn'] ?? 'N').toString().toUpperCase();
    final bool isReady = (myReadyYn == 'Y');
    final readyLabel = isReady ? '준비완료' : '준비';

    final btnStyle = ElevatedButton.styleFrom(
      backgroundColor: Colors.white,
      foregroundColor: Colors.black,
      side: const BorderSide(color: Colors.black, width: 1),
    );

    if (roomMasterYn == 'Y') {
      // 방장 => 3개
      return Row(
        children: [
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onOpenRoomSetting,
                child: const Text('방 설정'),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onToggleReady,
                child: Text(readyLabel),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 4),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onGameStart,
                child: const Text('게임 시작'),
              ),
            ),
          ),
        ],
      );
    } else {
      // 일반 => 2개
      return Row(
        children: [
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 8),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onOpenRoomSetting,
                child: const Text('방 설정'),
              ),
            ),
          ),
          Expanded(
            child: Container(
              margin: const EdgeInsets.symmetric(horizontal: 8),
              child: ElevatedButton(
                style: btnStyle,
                onPressed: _onToggleReady,
                child: Text(readyLabel),
              ),
            ),
          ),
        ],
      );
    }
  }

  Future<void> _onToggleReady() async {
    try {
      final me = _userList.firstWhere((u) => (u['user_seq'] ?? '0') == mySeq, orElse: () => {});
      final myReadyYn = (me['ready_yn'] ?? 'N').toString().toUpperCase();
      final isReady = (myReadyYn == 'Y');
      final newYn = isReady ? 'N' : 'Y';

      final userRef = _roomRef.child('userInfo').child(mySeq);
      await userRef.update({"ready_yn": newYn});
    } catch (e) {
      print('READY 설정 실패: $e');
    }
  }

  Future<void> _onOpenRoomSetting() async {
    final roomInfo = {
      "room_seq": "${widget.roomSeq}",
      "room_master_yn": roomMasterYn,
      "room_title": roomTitle,
      "room_intro": roomIntro,
      "open_yn": openYn,
      "room_pw": roomPw,
      "running_time": runningTime,
      "room_type": widget.roomType,
      "number_of_people": numberOfPeople,
      "score_open_range": scoreOpenRange,
    };

    final result = await showDialog(
      context: context,
      barrierDismissible: false,
      builder: (_) => RoomSettingModal(roomInfo: roomInfo),
    );
    if (result == 'refresh') {
      // ...
    }
  }

  Future<void> _onGameStart() async {
    final notReady = _userList.any((u) => (u['ready_yn'] ?? 'N').toString().toUpperCase() != 'Y');
    if (notReady) {
      showResponseDialog(context, '안내', 'READY되지 않은 참가자가 있습니다(방장 포함).');
      return;
    }

    final requestBody = {
      "room_seq": "${widget.roomSeq}",
      "room_type": "PRIVATE",
    };
    try {
      final response = await Api.serverRequest(uri: '/room/score/game/start', body: requestBody);
      if (response['result'] == 'OK') {
        final resp = response['response'] ?? {};
        if (resp['result'] == 'OK') {
          print('게임 시작 요청 성공(개인전)');
        } else {
          // ...
        }
      } else {
        // ...
      }
    } catch (e) {
      // ...
    }
  }

  // (★) 카운트다운 표시용
  String _formatDuration(Duration d) {
    final mm = d.inMinutes.remainder(60).toString().padLeft(2, '0');
    final ss = d.inSeconds.remainder(60).toString().padLeft(2, '0');
    return '$mm:$ss';
  }

  @override
  Widget build(BuildContext context) {
    // 남은시간 (기본: 60:00 ~ 0:00)
    final countdownStr = _formatDuration(_remaining);

    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        backgroundColor: Colors.black,
        elevation: 0,
        // 방 제목 + 남은시간 표시
        title: Text(
          (roomTitle.isNotEmpty ? roomTitle : '방 제목') + '  [$countdownStr]',
          style: const TextStyle(color: Colors.white),
        ),
        leading: IconButton(
          icon: const Icon(Icons.arrow_back_ios, color: Colors.white),
          onPressed: _onLeaveRoom,
        ),
      ),
      bottomNavigationBar: Container(
        height: 50,
        decoration: BoxDecoration(
          color: Colors.grey.shade300,
          border: Border.all(color: Colors.black, width: 1),
        ),
        child: const Center(
          child: Text('구글 광고', style: TextStyle(color: Colors.black)),
        ),
      ),
      body: _isLoading
          ? const Center(child: CircularProgressIndicator())
          : SingleChildScrollView(
              padding: const EdgeInsets.symmetric(vertical: 20, horizontal: 16),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  // 상단 버튼들
                  _buildTopButtons(),
                  const SizedBox(height: 20),

                  const Text('사회자', style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold)),
                  const SizedBox(height: 8),
                  _buildAdminSection(),
                  const SizedBox(height: 20),

                  const Text('참가자', style: TextStyle(fontSize: 16, fontWeight: FontWeight.bold)),
                  const SizedBox(height: 8),
                  _buildPlayerSection(),
                ],
              ),
            ),
    );
  }

  Widget _buildAdminSection() {
    final adminList = _userList.where((u) {
      final t = (u['participant_type'] ?? '').toString().toUpperCase();
      return t == 'ADMIN';
    }).toList();

    return Container(
      padding: const EdgeInsets.all(8),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: adminList.isEmpty
          ? const Text('사회자가 없습니다.')
          : Wrap(
              spacing: 16,
              runSpacing: 8,
              children: adminList.map(_buildSeat).toList(),
            ),
    );
  }

  Widget _buildPlayerSection() {
    final playerList = _userList.where((u) {
      final t = (u['participant_type'] ?? '').toString().toUpperCase();
      return t != 'ADMIN';
    }).toList();

    return Container(
      padding: const EdgeInsets.all(8),
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Colors.black),
        borderRadius: BorderRadius.circular(8),
      ),
      child: playerList.isEmpty
          ? const Text('참가자가 없습니다.')
          : SingleChildScrollView(
              child: Wrap(
                spacing: 16,
                runSpacing: 16,
                alignment: WrapAlignment.center,
                children: playerList.map(_buildSeat).toList(),
              ),
            ),
    );
  }

  Widget _buildSeat(Map<String, dynamic> userData) {
    final userName = userData['nickname'] ?? '유저';
    final profileImg = userData['profile_img'] ?? '';
    final readyYn = (userData['ready_yn'] ?? 'N').toString().toUpperCase();
    final connectYn = (userData['connect_yn'] ?? 'Y').toString().toUpperCase();
    final bool isReady = (readyYn == 'Y');
    final bool isDisconnected = (connectYn == 'N');
    final bool isMaster = (roomMasterYn == 'Y');

    return GestureDetector(
      onTap: () async {
        final result = await showDialog(
          context: context,
          barrierDismissible: false,
          builder: (_) => UserInfoPrivateDialog(
            userData: userData,
            isRoomMaster: isMaster,
            roomSeq: widget.roomSeq,
            roomTypeName: 'PRIVATE',
          ),
        );
        if (result == 'refresh') {
          // ...
        }
      },
      child: Container(
        margin: const EdgeInsets.only(right: 8),
        child: Column(
          children: [
            Container(
              width: 50,
              height: 50,
              decoration: BoxDecoration(
                color: Colors.white,
                border: Border.all(
                  color: isDisconnected
                      ? Colors.orange
                      : (isReady ? Colors.red : Colors.black),
                  width: isDisconnected ? 2 : (isReady ? 2 : 1),
                ),
                borderRadius: BorderRadius.circular(20),
                boxShadow: [
                  if (isReady)
                    BoxShadow(
                      color: Colors.redAccent.withOpacity(0.6),
                      blurRadius: 8,
                      spreadRadius: 2,
                    ),
                  if (isDisconnected)
                    BoxShadow(
                      color: Colors.orangeAccent.withOpacity(0.6),
                      blurRadius: 8,
                      spreadRadius: 2,
                    ),
                ],
              ),
              child: ClipRRect(
                borderRadius: BorderRadius.circular(20),
                child: isDisconnected
                    ? const Center(
                        child: Text(
                          '!',
                          style: TextStyle(
                            fontSize: 20,
                            color: Colors.orange,
                            fontWeight: FontWeight.bold,
                          ),
                        ),
                      )
                    : Image.network(
                        'https://eldsoft.com:8097/images$profileImg',
                        fit: BoxFit.cover,
                        errorBuilder: (_, __, ___) => const Center(
                          child: Text(
                            '이미지\n불가',
                            textAlign: TextAlign.center,
                            style: TextStyle(fontSize: 10),
                          ),
                        ),
                      ),
              ),
            ),
            const SizedBox(height: 4),
            Text(userName, style: const TextStyle(fontSize: 12)),
          ],
        ),
      ),
    );
  }
}
